//g++ -o ./bin/icon-dm ./src/icon-dm.c `pkg-config --cflags --libs glib-2.0` -lImlib2 -lXft -lX11 -lfreetype -lXrender -lfontconfig -lpng12 -lz -lm -lcrypt -lXmu -lpng -ljpeg -lrt -Wall -I. -I/usr/include/freetype2 -I/usr/include/freetype2/config -I/usr/include/libpng12 -I/usr/include -lpam

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <assert.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xatom.h>
#include <glib-2.0/glib.h>
#include <grp.h>
#include <pwd.h>
#include <sys/types.h>
#include <fcntl.h>
#include <signal.h>
#include <errno.h>
#include <sys/stat.h>
#include <shadow.h>
#include <X11/cursorfont.h>
#include <syslog.h>
#include <pthread.h>

#include "./other_src/rc_str.c"
#include "./other_src/button.c"

imenu m_menu;

Visual                         *vis;
Colormap                       cm;
int                            depth;
XSetWindowAttributes           attr;
char cmd[256];
//static gboolean               envir_utf8; //2012.04.18
//****************************************************************

#include <X11/Xlib.h>
#include <X11/Xos.h>
#include <X11/Xatom.h>
#include <stdio.h>
#include <ctype.h>
#include <stdint.h>

#include <signal.h>
#include <sys/wait.h>
#include <errno.h>
#include <setjmp.h>
#include <stdarg.h>

#ifdef __APPLE__
#include <AvailabilityMacros.h>
#if MAC_OS_X_VERSION_MIN_REQUIRED >= 1060
#include <vproc.h>
#endif
#endif

/* For PRIO_PROCESS and setpriority() */
#include <sys/time.h>
#include <sys/resource.h>

#include <stdlib.h>

#ifndef SHELL
#define SHELL "sh"
#endif

const char *bindir = "/usr/bin/";
const char * const server_names[] = {
#ifdef __APPLE__
    "Xquartz     Mac OSX Quartz displays.",
#else
# ifdef __CYGWIN__
    "XWin        X Server for the Cygwin environment on Microsoft Windows",
# else
    "Xorg        Common X server for most displays",
# endif
#endif
    "Xvfb        Virtual frame buffer",
    "Xfake       kdrive-based virtual frame buffer",
    "Xnest       X server nested in a window on another X server",
    "Xephyr      kdrive-based nested X server",
    "Xvnc        X server accessed over VNC's RFB protocol",
    "Xdmx        Distributed Multi-head X server",
    NULL};

#ifndef XINITRC
#define XINITRC ".xinitrc"
#endif
char xinitrcbuf[256];

#ifndef XSERVERRC
#define XSERVERRC ".xserverrc"
#endif
char xserverrcbuf[256];

static char default_server[128] = "X";
static char default_display[128] = ":0";        /* choose most efficient */
static char *serverargv[100];
static char **server = serverargv + 2;        /* make sure room for sh .xserverrc args */
static char *displayNum = NULL;
static char *program = NULL;
static Display *xd = NULL;            /* server connection */
int status;
int serverpid = -1;
int clientpid = -1;
volatile int gotSignal = 0;

static void Execute(char **vec);
static Bool waitforserver(void);
static Bool processTimeout(int timeout, char *string);
static int startServer(char *server[]);
static int ignorexio(Display *dpy);
static void shutdown(void);

static void Fatal(const char *fmt, ...);
static void Error(const char *fmt, ...);
static void Fatalx(const char *fmt, ...);
static void Errorx(const char *fmt, ...);

static void
sigCatch(int sig)
{
    /* On system with POSIX signals, just interrupt the system call */
    gotSignal = sig;
}

static void
sigIgnore(int sig)
{
}

static void
Execute(char **vec)        /* has room from up above */
{
    execvp(vec[0], vec);
    if (access(vec[0], R_OK) == 0) {
    vec--;                /* back it up to stuff shell in */
    vec[0] =(char*) SHELL;
    execvp(vec[0], vec);
    }
    return;
}

int initserver(int argc, char *argv[])
{
    register char **sptr = server;
    int server_given = 0;
    int server_args_given = 0;
    int start_of_server_args;
    struct sigaction sa, si;
#ifdef __APPLE__
#if MAC_OS_X_VERSION_MIN_REQUIRED >= 1060
    vproc_transaction_t vt;
#endif
#endif

    program = *argv++;
    argc--;
    if (argc) {
        argv++;
        argc--;
    }

    if (argc == 0 ||
        (**argv != '/' && **argv != '.')) {
        *sptr++ = default_server;
    } else {
        server_given = 1;
        *sptr++ = *argv++;
        argc--;
    }
    if (argc > 0 && (argv[0][0] == ':' && isdigit(argv[0][1])))
        displayNum = *argv;
    else
        displayNum = *sptr++ = default_display;

    start_of_server_args = (sptr - server);
    while (--argc >= 0) {
        server_args_given++;
        *sptr++ = *argv++;
    }
    *sptr = NULL;
    if (!server_given) {
        char *cp;
        Bool required = False;

        xserverrcbuf[0] = '\0';
        if ((cp = getenv("XSERVERRC")) != NULL) {
            snprintf(xserverrcbuf, sizeof(xserverrcbuf), "%s", cp);
            required = True;
        } else if ((cp = getenv("HOME")) != NULL) {
            snprintf(xserverrcbuf, sizeof(xserverrcbuf),
                     "%s/%s", cp, XSERVERRC);
        }
        if (xserverrcbuf[0]) {
            if (access(xserverrcbuf, F_OK) == 0) {
                server += start_of_server_args - 1;
                server[0] = xserverrcbuf;
            } else if (required) {
                Error("warning, no server init file \"%s\"", xserverrcbuf);
            }
        }
    }

    signal(SIGCHLD, SIG_DFL); 

    memset(&sa, 0, sizeof sa);
    sa.sa_handler = sigCatch;
    sigemptyset(&sa.sa_mask);
    sa.sa_flags = SA_SIGINFO;

    sigaction (SIGCHLD, &sa, NULL);
    sigaction(SIGTERM, &sa, NULL);
    sigaction(SIGQUIT, &sa, NULL);
    sigaction(SIGINT, &sa, NULL);
    sigaction(SIGHUP, &sa, NULL);
    sigaction(SIGPIPE, &sa, NULL);

    memset(&si, 0, sizeof(si));
    si.sa_handler = sigIgnore;
    sigemptyset(&si.sa_mask);
    si.sa_flags = SA_RESTART;

    sigaction(SIGALRM, &si, NULL);
    sigaction(SIGUSR1, &si, NULL);

#ifdef __APPLE__
#if MAC_OS_X_VERSION_MIN_REQUIRED >= 1060
    vt = vproc_transaction_begin(NULL);
#endif
#endif

    if (startServer(server) > 0) {
        setenv("DISPLAY",displayNum,1);
        return 1;
    }
}

void *exe_cmd(void * cmd)
{
    Display                        *disp;
    disp = XOpenDisplay(displayNum);
    if (disp == NULL)
    {
       printf("Can't open display %s\n", displayNum);
    }
    FILE *fp;
    if ((fp=popen((char *)cmd,"r"))==NULL)
    {
        printf("error: %s/n", strerror(errno));
    }
    else
    {
        pclose(fp);
    }
    setenv("SIG","END",1);
}


int main(int argc, char *argv[])
{
//**********************************************************************************
    initserver(argc,argv);
    printf("%s\n",displayNum);
    Window                         win;
//    int                                 w, h;
    XEvent                            ev;
    Display                        *disp;
    disp = XOpenDisplay(displayNum);
    if (disp == NULL)
    {
       printf("Can't open display %s\n", displayNum);
       return 1;
    }
    Window grabWin = DefaultRootWindow(disp);
    Cursor grab_good =XCreateFontCursor(disp, XC_arrow);
    XDefineCursor(disp,grabWin,grab_good);
    int                 w, h;
    Imlib_Image         back;
    Imlib_Color_Range   range;
    Imlib_Image         buffer;
    int scr_screen = DefaultScreen(disp);
    int scr_height = DisplayHeight(disp, scr_screen);
    int scr_width = DisplayWidth(disp, scr_screen);
//    uid_t uid;

    vis = DefaultVisual(disp, DefaultScreen(disp));
    depth = DefaultDepth(disp, DefaultScreen(disp));
    cm = DefaultColormap(disp, DefaultScreen(disp));
        // text
    range = imlib_create_color_range();
    imlib_context_set_color_range(range);
    imlib_context_set_color(255, 255, 255, 255);
    imlib_add_color_to_color_range(0);
    /* add an orange color, semi-transparent 10 units from the first */
    imlib_context_set_color(255, 200, 10, 255);
    imlib_add_color_to_color_range(10);
    /* add black, fully transparent at the end 20 units away */
    imlib_context_set_color(70, 0, 0, 255);
    imlib_add_color_to_color_range(20);

    attr.override_redirect=True;
    win=XCreateWindow(disp,DefaultRootWindow(disp),0,0,scr_width,scr_height,0,0,0,CopyFromParent,CWBackingStore|CWOverrideRedirect|CWEventMask,&attr);
    XSelectInput(disp, win, ButtonPressMask | ButtonReleaseMask | ButtonMotionMask | PointerMotionMask | ExposureMask | EnterWindowMask | LeaveWindowMask);
    XMapWindow(disp, win);
    XLowerWindow(disp,win);
    imlib_context_set_display(disp);
    imlib_context_set_visual(vis);
    imlib_context_set_colormap(cm);
    imlib_context_set_drawable(win);
    imlib_context_set_dither(1);
    imlib_context_set_blend(0);
    imlib_context_set_color_modifier(NULL);
     /**
        * Start rendering
        */
    int i;
    init_button(m_menu);
    int n=0;
    struct passwd *pw;
    FILE *freeNum;
    char buffer_rc[1024];
//    uid = getuid ();
    setpwent();   
    pw = getpwent();
    n=0;
    while (pw)
    {
        if ((pw->pw_uid>=1000) and (pw->pw_uid<65534))
        {
            printf("%s %d\n",pw->pw_name,pw->pw_uid);
            n=n+1;
        }
        pw = getpwent();
    }
    endpwent();
    i=0;
    
    m_menu.t_group=0;
    m_menu.show_lines=11;
    m_menu.x=100;
    m_menu.y=60;
    m_menu.width1=800;
    m_menu.width2=120;
    m_menu.width3=80;
    m_menu.width4=269;
    m_menu.width_text=800;
    m_menu.height=25;
    m_menu.w_gap=3;
    m_menu.h_gap=15;
    m_menu.b_up.hide=1;
    m_menu.b_down.hide=1;
    m_menu.m_group_begin=0;
    m_menu.m_group_end=9;
    m_menu.menu1.flash=1;
    m_menu.menu1.group=-1;
    m_menu.menu1.hide=0;
    m_menu.menu1.selected=0;
    m_menu.menu4[0].hide=0;
    m_menu.menu4[1].hide=0;
    m_menu.menu4[2].hide=0;
    sprintf(m_menu.menu4[0].text,"登录");
    sprintf(m_menu.menu4[1].text,"关机");
    sprintf(m_menu.menu4[2].text,"重启");
    sprintf(m_menu.menu1.text,"请分别选择以下选项，然后按“确认”。");
    
    sprintf(m_menu.input.text,"请在此录入密码。");
    m_menu.input.is_passwd=1;
    m_menu.input.tag_text[0]='\0';
    m_menu.input.hide=0;
    m_menu.input.w_id=0;
    m_menu.input.str_len=0;

    m_menu.menu2[0].flash=0;
    m_menu.menu2[0].group=0;
    m_menu.menu2[0].hide=0;
    m_menu.menu2[0].selected=0;
    sprintf(m_menu.menu2[0].text,"各种 DE 环境：");
    
    m_menu.menu3[0].flash=0;
    m_menu.menu3[0].group=0;
    m_menu.menu3[0].hide=0;
    m_menu.menu3[0].selected=0;
    sprintf(m_menu.menu3[0].text,"unity");
    sprintf(m_menu.menu3[0].tag_text,"unity");
    
    m_menu.menu3[1].flash=0;
    m_menu.menu3[1].group=0;
    m_menu.menu3[1].hide=0;
    m_menu.menu3[1].selected=0;
    sprintf(m_menu.menu3[1].text,"gnome-shell");
    sprintf(m_menu.menu3[1].tag_text,"gnome-shell");
    
    m_menu.menu3[2].flash=0;
    m_menu.menu3[2].group=0;
    m_menu.menu3[2].hide=0;
    m_menu.menu3[2].selected=0;
    sprintf(m_menu.menu3[2].text,"kde");
    sprintf(m_menu.menu3[2].tag_text,"startkde");
    
    m_menu.menu3[3].flash=0;
    m_menu.menu3[3].group=0;
    m_menu.menu3[3].hide=0;
    m_menu.menu3[3].selected=0;
    sprintf(m_menu.menu3[3].text,"xfce4");
    sprintf(m_menu.menu3[3].tag_text,"startxfce4");
    
    m_menu.menu3[4].flash=0;
    m_menu.menu3[4].group=0;
    m_menu.menu3[4].hide=0;
    m_menu.menu3[4].selected=0;
    sprintf(m_menu.menu3[4].text,"lxde");
    sprintf(m_menu.menu3[4].tag_text,"startlxde");
    
    m_menu.menu3[5].flash=0;
    m_menu.menu3[5].group=0;
    m_menu.menu3[5].hide=0;
    m_menu.menu3[5].selected=0;
    sprintf(m_menu.menu3[5].text,"icon-de");
    sprintf(m_menu.menu3[5].tag_text,"icon-de");

    m_menu.menu2[1].flash=0;
    m_menu.menu2[1].group=1;
    m_menu.menu2[1].hide=0;
    m_menu.menu2[1].selected=0;
    sprintf(m_menu.menu2[1].text," box 类 ：");
    
    m_menu.menu3[10].flash=0;
    m_menu.menu3[10].group=1;
    m_menu.menu3[10].hide=0;
    m_menu.menu3[10].selected=0;
    sprintf(m_menu.menu3[10].text,"fluxbox");
    sprintf(m_menu.menu3[10].tag_text,"startfluxbox");
    
    m_menu.menu3[11].flash=0;
    m_menu.menu3[11].group=1;
    m_menu.menu3[11].hide=0;
    m_menu.menu3[11].selected=0;
    sprintf(m_menu.menu3[11].text,"blackbox");
    sprintf(m_menu.menu3[11].tag_text,"blackbox");
    
    m_menu.menu3[12].flash=0;
    m_menu.menu3[12].group=1;
    m_menu.menu3[12].hide=0;
    m_menu.menu3[12].selected=0;
    sprintf(m_menu.menu3[12].text,"Openbox");
    sprintf(m_menu.menu3[12].tag_text,"openbox");
    
    m_menu.menu3[13].flash=0;
    m_menu.menu3[13].group=1;
    m_menu.menu3[13].hide=0;
    m_menu.menu3[13].selected=0;
    sprintf(m_menu.menu3[13].text,"Sapphire");
    sprintf(m_menu.menu3[13].tag_text,"sapphire");

    m_menu.menu2[2].flash=0;
    m_menu.menu2[2].group=2;
    m_menu.menu2[2].hide=0;
    m_menu.menu2[2].selected=0;
    sprintf(m_menu.menu2[2].text," 知名/常见 WM：");
    
    m_menu.menu3[20].flash=0;
    m_menu.menu3[20].group=2;
    m_menu.menu3[20].hide=0;
    m_menu.menu3[20].selected=0;
    sprintf(m_menu.menu3[20].text,"e17");
    sprintf(m_menu.menu3[20].tag_text,"enlightenment_start");
    
    m_menu.menu3[21].flash=0;
    m_menu.menu3[21].group=2;
    m_menu.menu3[21].hide=0;
    m_menu.menu3[21].selected=0;
    sprintf(m_menu.menu3[21].text,"compiz");
    sprintf(m_menu.menu3[21].tag_text,"compiz");
    
    m_menu.menu3[22].flash=0;
    m_menu.menu3[22].group=2;
    m_menu.menu3[22].hide=0;
    m_menu.menu3[22].selected=0;
    sprintf(m_menu.menu3[22].text,"sawfish");
    sprintf(m_menu.menu3[22].tag_text,"sawfish");
    
    m_menu.menu3[23].flash=0;
    m_menu.menu3[23].group=2;
    m_menu.menu3[23].hide=0;
    m_menu.menu3[23].selected=0;
    sprintf(m_menu.menu3[23].text,"pekwm");
    sprintf(m_menu.menu3[23].tag_text,"pekwm");
    
    m_menu.menu3[24].flash=0;
    m_menu.menu3[24].group=2;
    m_menu.menu3[24].hide=0;
    m_menu.menu3[24].selected=0;
    sprintf(m_menu.menu3[24].text,"Jwm");
    sprintf(m_menu.menu3[24].tag_text,"jwm");
    
    m_menu.menu3[25].flash=0;
    m_menu.menu3[25].group=2;
    m_menu.menu3[25].hide=0;
    m_menu.menu3[25].selected=0;
    sprintf(m_menu.menu3[25].text,"Icewm");
    sprintf(m_menu.menu3[25].tag_text,"icewm");
    
    m_menu.menu3[26].flash=0;
    m_menu.menu3[26].group=2;
    m_menu.menu3[26].hide=0;
    m_menu.menu3[26].selected=0;
    sprintf(m_menu.menu3[26].text,"oroborus");
    sprintf(m_menu.menu3[26].tag_text,"oroborus");
    
    m_menu.menu3[27].flash=0;
    m_menu.menu3[27].group=2;
    m_menu.menu3[27].hide=0;
    m_menu.menu3[27].selected=0;
    sprintf(m_menu.menu3[27].text,"Flwm");
    sprintf(m_menu.menu3[27].tag_text,"flwm");

    m_menu.menu2[3].flash=0;
    m_menu.menu2[3].group=3;
    m_menu.menu2[3].hide=0;
    m_menu.menu2[3].selected=0;
    sprintf(m_menu.menu2[3].text,"Wmaker类：");
    
    m_menu.menu3[30].flash=0;
    m_menu.menu3[30].group=3;
    m_menu.menu3[30].hide=0;
    m_menu.menu3[30].selected=0;
    sprintf(m_menu.menu3[30].text,"Wmaker");
    sprintf(m_menu.menu3[30].tag_text,"wmaker");
    
    m_menu.menu3[31].flash=0;
    m_menu.menu3[31].group=3;
    m_menu.menu3[31].hide=0;
    m_menu.menu3[31].selected=0;
    sprintf(m_menu.menu3[31].text,"AfterStep");
    sprintf(m_menu.menu3[31].tag_text,"afterStep");

    m_menu.menu2[4].flash=0;
    m_menu.menu2[4].group=4;
    m_menu.menu2[4].hide=0;
    m_menu.menu2[4].selected=0;
    sprintf(m_menu.menu2[4].text,"虚拟视窗类：");
    
    m_menu.menu3[40].flash=0;
    m_menu.menu3[40].group=4;
    m_menu.menu3[40].hide=0;
    m_menu.menu3[40].selected=0;
    sprintf(m_menu.menu3[40].text,"fvwm");
    sprintf(m_menu.menu3[40].tag_text,"fvwm");
    
    m_menu.menu3[41].flash=0;
    m_menu.menu3[41].group=4;
    m_menu.menu3[41].hide=0;
    m_menu.menu3[41].selected=0;
    sprintf(m_menu.menu3[41].text,"olvwm");
    sprintf(m_menu.menu3[41].tag_text,"olvwm");
    
    m_menu.menu3[42].flash=0;
    m_menu.menu3[42].group=4;
    m_menu.menu3[42].hide=0;
    m_menu.menu3[42].selected=0;
    sprintf(m_menu.menu3[42].text,"VTWM");
    sprintf(m_menu.menu3[42].tag_text,"vtwm");
    
    m_menu.menu2[5].flash=0;
    m_menu.menu2[5].group=5;
    m_menu.menu2[5].hide=0;
    m_menu.menu2[5].selected=0;
    sprintf(m_menu.menu2[5].text," 平铺 类：");
    
    m_menu.menu3[50].flash=0;
    m_menu.menu3[50].group=5;
    m_menu.menu3[50].hide=0;
    m_menu.menu3[50].selected=0;
    sprintf(m_menu.menu3[50].text,"ratpoison");
    sprintf(m_menu.menu3[50].tag_text,"ratpoison");
    
    m_menu.menu3[51].flash=0;
    m_menu.menu3[51].group=5;
    m_menu.menu3[51].hide=0;
    m_menu.menu3[51].selected=0;
    sprintf(m_menu.menu3[51].text,"StumpWM");
    sprintf(m_menu.menu3[51].tag_text,"stumpwm");
    
    m_menu.menu3[52].flash=0;
    m_menu.menu3[52].group=5;
    m_menu.menu3[52].hide=0;
    m_menu.menu3[52].selected=0;
    sprintf(m_menu.menu3[52].text,"awesome");
    sprintf(m_menu.menu3[52].tag_text,"awesome");
    
    m_menu.menu3[53].flash=0;
    m_menu.menu3[53].group=5;
    m_menu.menu3[53].hide=0;
    m_menu.menu3[53].selected=0;
    sprintf(m_menu.menu3[53].text,"i3");
    sprintf(m_menu.menu3[53].tag_text,"i3");
    
    m_menu.menu3[54].flash=0;
    m_menu.menu3[54].group=5;
    m_menu.menu3[54].hide=0;
    m_menu.menu3[54].selected=0;
    sprintf(m_menu.menu3[54].text,"dwm");
    sprintf(m_menu.menu3[54].tag_text,"dwm");
    
    m_menu.menu3[55].flash=0;
    m_menu.menu3[55].group=5;
    m_menu.menu3[55].hide=0;
    m_menu.menu3[55].selected=0;
    sprintf(m_menu.menu3[55].text,"wmii");
    sprintf(m_menu.menu3[55].tag_text,"wmii");
    
    m_menu.menu3[56].flash=0;
    m_menu.menu3[56].group=5;
    m_menu.menu3[56].hide=0;
    m_menu.menu3[56].selected=0;
    sprintf(m_menu.menu3[56].text,"Bluetile");
    sprintf(m_menu.menu3[56].tag_text,"bluetile");

    m_menu.menu2[6].flash=0;
    m_menu.menu2[6].group=6;
    m_menu.menu2[6].hide=0;
    m_menu.menu2[6].selected=0;
    sprintf(m_menu.menu2[6].text," 微型 WM：");
    
    m_menu.menu3[60].flash=0;
    m_menu.menu3[60].group=6;
    m_menu.menu3[60].hide=0;
    m_menu.menu3[60].selected=0;
    sprintf(m_menu.menu3[60].text,"ctwm");
    sprintf(m_menu.menu3[60].tag_text,"ctwm");
    
    m_menu.menu3[61].flash=0;
    m_menu.menu3[61].group=6;
    m_menu.menu3[61].hide=0;
    m_menu.menu3[61].selected=0;
    sprintf(m_menu.menu3[61].text,"LWM");
    sprintf(m_menu.menu3[61].tag_text,"lwm");
    
    m_menu.menu3[62].flash=0;
    m_menu.menu3[62].group=6;
    m_menu.menu3[62].hide=0;
    m_menu.menu3[62].selected=0;
    sprintf(m_menu.menu3[62].text,"MIWM");
    sprintf(m_menu.menu3[62].tag_text,"miwm");
    
    m_menu.menu3[63].flash=0;
    m_menu.menu3[63].group=6;
    m_menu.menu3[63].hide=0;
    m_menu.menu3[63].selected=0;
    sprintf(m_menu.menu3[63].text,"TinyWM");
    sprintf(m_menu.menu3[63].tag_text,"tinywm");
    
    m_menu.menu3[64].flash=0;
    m_menu.menu3[64].group=6;
    m_menu.menu3[64].hide=0;
    m_menu.menu3[64].selected=0;
    sprintf(m_menu.menu3[64].text,"WindowLab");
    sprintf(m_menu.menu3[64].tag_text,"WindowLab");
    
    m_menu.menu3[65].flash=0;
    m_menu.menu3[65].group=6;
    m_menu.menu3[65].hide=0;
    m_menu.menu3[65].selected=0;
    sprintf(m_menu.menu3[65].text,"wm2");
    sprintf(m_menu.menu3[65].tag_text,"wm2");
    
    m_menu.menu3[66].flash=0;
    m_menu.menu3[66].group=6;
    m_menu.menu3[66].hide=0;
    m_menu.menu3[66].selected=0;
    sprintf(m_menu.menu3[66].text,"evilwm");
    sprintf(m_menu.menu3[66].tag_text,"evilwm");

    m_menu.menu2[7].flash=0;
    m_menu.menu2[7].group=7;
    m_menu.menu2[7].hide=0;
    m_menu.menu2[7].selected=0;
    sprintf(m_menu.menu2[7].text," 其他老旧WM：");
    
    m_menu.menu3[70].flash=0;
    m_menu.menu3[70].group=7;
    m_menu.menu3[70].hide=0;
    m_menu.menu3[70].selected=0;
    sprintf(m_menu.menu3[70].text,"9wm");
    sprintf(m_menu.menu3[70].tag_text,"9wm");
    
    m_menu.menu3[71].flash=0;
    m_menu.menu3[71].group=7;
    m_menu.menu3[71].hide=0;
    m_menu.menu3[71].selected=0;
    sprintf(m_menu.menu3[71].text,"aewm");
    sprintf(m_menu.menu3[71].tag_text,"aewm");
    
    m_menu.menu3[72].flash=0;
    m_menu.menu3[72].group=7;
    m_menu.menu3[72].hide=0;
    m_menu.menu3[72].selected=0;
    sprintf(m_menu.menu3[72].text,"aewm++");
    sprintf(m_menu.menu3[72].tag_text,"aewm++");
    
    m_menu.menu3[73].flash=0;
    m_menu.menu3[73].group=7;
    m_menu.menu3[73].hide=0;
    m_menu.menu3[73].selected=0;
    sprintf(m_menu.menu3[73].text,"olwm");
    sprintf(m_menu.menu3[73].tag_text,"olwm");
    
    m_menu.menu3[74].flash=0;
    m_menu.menu3[74].group=7;
    m_menu.menu3[74].hide=0;
    m_menu.menu3[74].selected=0;
    sprintf(m_menu.menu3[74].text,"w9wm");
    sprintf(m_menu.menu3[74].tag_text,"w9wm");
    
    m_menu.menu3[75].flash=0;
    m_menu.menu3[75].group=7;
    m_menu.menu3[75].hide=0;
    m_menu.menu3[75].selected=0;
    sprintf(m_menu.menu3[75].text,"Scrotwm");
    sprintf(m_menu.menu3[75].tag_text,"scrotwm");
    
    m_menu.menu3[76].flash=0;
    m_menu.menu3[76].group=7;
    m_menu.menu3[76].hide=0;
    m_menu.menu3[76].selected=0;
    sprintf(m_menu.menu3[76].text,"tritium");
    sprintf(m_menu.menu3[76].tag_text,"tritium");
    
    m_menu.menu3[77].flash=0;
    m_menu.menu3[77].group=7;
    m_menu.menu3[77].hide=0;
    m_menu.menu3[77].selected=0;
    sprintf(m_menu.menu3[77].text,"amiwm");
    sprintf(m_menu.menu3[77].tag_text,"amiwm");
    
    m_menu.menu2[8].flash=0;
    m_menu.menu2[8].group=8;
    m_menu.menu2[8].hide=0;
    m_menu.menu2[8].selected=0;
    sprintf(m_menu.menu2[8].text," DE中的WM：");
    
    m_menu.menu3[80].flash=0;
    m_menu.menu3[80].group=8;
    m_menu.menu3[80].hide=0;
    m_menu.menu3[80].selected=0;
    sprintf(m_menu.menu3[80].text,"compiz");
    sprintf(m_menu.menu3[80].tag_text,"compiz");
    
    m_menu.menu3[81].flash=0;
    m_menu.menu3[81].group=8;
    m_menu.menu3[81].hide=0;
    m_menu.menu3[81].selected=0;
    sprintf(m_menu.menu3[81].text,"mutter");
    sprintf(m_menu.menu3[81].tag_text,"mutter");
    
    m_menu.menu3[82].flash=0;
    m_menu.menu3[82].group=8;
    m_menu.menu3[82].hide=0;
    m_menu.menu3[82].selected=0;
    sprintf(m_menu.menu3[82].text,"metacity");
    sprintf(m_menu.menu3[82].tag_text,"metacity");
    
    m_menu.menu3[83].flash=0;
    m_menu.menu3[83].group=8;
    m_menu.menu3[83].hide=0;
    m_menu.menu3[83].selected=0;
    sprintf(m_menu.menu3[83].text,"kwin");
    sprintf(m_menu.menu3[83].tag_text,"kwin");
    
    m_menu.menu3[84].flash=0;
    m_menu.menu3[84].group=8;
    m_menu.menu3[84].hide=0;
    m_menu.menu3[84].selected=0;
    sprintf(m_menu.menu3[84].text,"xfwm4");
    sprintf(m_menu.menu3[84].tag_text,"xfwm4");
    m_menu.menu2[10].flash=0;
    m_menu.menu2[10].group=10;
    m_menu.menu2[10].hide=0;
    m_menu.menu2[10].selected=0;
    sprintf(m_menu.menu2[10].text,"操 作 员：");

    setpwent();   
    pw = getpwent();
    i=100;
    while (pw)
    {
        if ((pw->pw_uid>=1000) and (pw->pw_uid<65534))
        {
            m_menu.menu3[i].flash=0;
            m_menu.menu3[i].group=10;
            m_menu.menu3[i].hide=0;
            m_menu.menu3[i].selected=0;
            sprintf(m_menu.menu3[i].text,"%s",pw->pw_name);
            sprintf(m_menu.menu3[i].tag_text,"%s",pw->pw_passwd);
            m_menu.menu3[i].tag_int=pw->pw_uid;
            i=i+1;
        }
        pw = getpwent();
    }
    endpwent();
    char background[1024];
    if (access("/etc/icon-de/icon-dm.rc",0)==-1)
    {
        sprintf(m_menu.menu1.text,"请分别选择以下选项，然后按“确认”——没有找到配置文件，不能恢复上次的选择。");
    }
    else
    {

        freeNum=fopen("/etc/icon-de/icon-dm.rc","r");
        if ( freeNum==NULL)
        { 
            printf("error on open /etc/icon-de/icon-dm.rc\n"); 
        }
        else
        {
            m_menu.menu2[9].flash=0;
            m_menu.menu2[9].group=9;
            m_menu.menu2[9].hide=0;
            m_menu.menu2[9].selected=0;
            sprintf(m_menu.menu2[9].text," 自定义WM：");
            char usr[1024],wm[1024],ch2[1024];
            while ( ! feof(freeNum) )
            {
                fgets(buffer_rc,1024,freeNum);
                for (n=0;n<1024;n++)
                    if (buffer_rc[n]=='\n') buffer_rc[n]='\0';
        //        printf("%s\n",buffer_rc); 
        //        sprintf(command,"");
                if (strcmp(buffer_rc,"")!=0 )
                {
                    sprintf(ch2,"%c",buffer_rc[0]);
                    if (strcmp(ch2,"#")!=0 ) 
                    {
                        if (check_rc(buffer_rc,"our_wm1:")==0 )
                        {
                            get_str(wm,buffer_rc,"our_wm1:");
                            m_menu.menu3[90].flash=0;
                            m_menu.menu3[90].group=9;
                            m_menu.menu3[90].hide=0;
                            m_menu.menu3[90].selected=0;
                            sprintf(m_menu.menu3[90].text,"%s",wm);
                            sprintf(m_menu.menu3[90].tag_text,"%s",wm);
                        }
                        if (check_rc(buffer_rc,"our_wm2:")==0 )
                        {
                            get_str(wm,buffer_rc,"our_wm2:");
                            m_menu.menu3[91].flash=0;
                            m_menu.menu3[91].group=9;
                            m_menu.menu3[91].hide=0;
                            m_menu.menu3[91].selected=0;
                            sprintf(m_menu.menu3[91].text,"%s",wm);
                            sprintf(m_menu.menu3[91].tag_text,"%s",wm);
                        }
                        if (check_rc(buffer_rc,"our_wm3:")==0 )
                        {
                            get_str(wm,buffer_rc,"our_wm3:");
                            m_menu.menu3[92].flash=0;
                            m_menu.menu3[92].group=9;
                            m_menu.menu3[92].hide=0;
                            m_menu.menu3[92].selected=0;
                            sprintf(m_menu.menu3[92].text,"%s",wm);
                            sprintf(m_menu.menu3[92].tag_text,"%s",wm);
                        }
                        if (check_rc(buffer_rc,"background:")==0 )
                        {
                            get_str(background,buffer_rc,"background:");
                        }
                        if (check_rc(buffer_rc,"default_wm:")==0 )
                        {
                            get_str(wm,buffer_rc,"default_wm:");
                            for (n=0;n<99;n++)
                            {
                                if (strcmp(wm,m_menu.menu3[n].text) == 0)
                                {
                                    m_menu.menu3[n].selected=1;
                                    break;
                                }
                            }
                        }
                        if (check_rc(buffer_rc,"default_usr:")==0 )
                        {
                            get_str(usr,buffer_rc,"default_usr:");
                            for (n=100;n<500;n++)
                            {
                                if (strcmp(usr,m_menu.menu3[n].text) == 0)
                                {
                                    m_menu.menu3[n].selected=1;
                                    break;
                                }
                            }
                        }
                        
                    }     
                }    
            }
            fclose(freeNum);
            sprintf(m_menu.menu1.text,"请分别选择以下选项，然后按“登录”——上次的选择:WM--%s,USER--%s",wm,usr);
        }
        
    }

    w=scr_width;
    h=scr_height;
    back = imlib_load_image(background);
    if (back!=NULL)
    {
        imlib_context_set_image(back);
        XMapWindow(disp, win);
        imlib_context_set_image(back);
        imlib_context_set_blend(1);
        imlib_context_set_image(back);
        int ww,hh;
        ww=imlib_image_get_width();
        hh=imlib_image_get_height();
        for (i=0;i<scr_width;i=i+ww)
        {
            for (n=0;n<scr_width;n=n+hh)
            {
                imlib_render_image_on_drawable(i, n);
            }
        }
        buffer = imlib_create_cropped_scaled_image(0, 0,imlib_image_get_width() ,imlib_image_get_height() , w , h);
    }
    else
    {
        printf("can't open background:%s \n",background);
    }
    XResizeWindow(disp, win, w, h);
    XSync(disp, False);
    
    creat_button(disp,m_menu);
    XSetInputFocus(disp, m_menu.input.w_ip, RevertToParent, CurrentTime);
    int xfd = ConnectionNumber(disp);
    fd_set rfds;
    struct timeval tm;
    int r_flash=0;
    int buf_len=0,len;
    char *k_buffer = (char *)malloc(buf_len * sizeof(char));
    KeySym keysym;
    printf("while_begin\n");
    putenv((char *)"SIG=NULL");//标记是否有子进程在运行的环境变量。
    while (1)
    {
        if (XPending(disp))
        {
            XNextEvent(disp, &ev);
            if (XFilterEvent(&ev, None))
                continue;
            switch (ev.type)
            {
            case Expose:
                break;
            case MotionNotify:
                break;
            default:
                break;
            case EnterNotify:
                break;
            case KeyPress:
                len = XLookupString((XKeyPressedEvent*)&ev, k_buffer, buf_len,
                              &keysym, 0);
                if (keysym == XK_Return)
                {
                    i=5;
//                    char passwd[1024];
                    for (n=100;n<500;n++)
                    {
                        if (m_menu.menu3[n].selected==1)
                            break;
                    }
//                    char *name, pwname[9];
                    int shadow = 0;
                    int error=0;
                    int pwd_good=0;
                    struct spwd *spw;
                    pw = getpwuid(m_menu.menu3[n].tag_int);
                    if (pw == NULL) {
                        error=1;
                    }
                    printf("Password:%s\n",pw->pw_passwd);
                    if ((pw->pw_passwd[0] == 'x') or (pw->pw_passwd[1] == 'x') )
                    {
                        shadow = 1;
                        spw=getspnam(pw->pw_name);
                        if (spw == NULL)
                            error=1;
                        else
                            printf("Password:%s\n",spw->sp_pwdp);
                    } 
                    if (shadow == 1)
                    {
                        printf("spw:%s|||\n",spw->sp_pwdp);
                        printf("spw:%s|||\n",crypt(m_menu.input.tag_text, spw->sp_pwdp));
                        if (strcmp(crypt(m_menu.input.tag_text, spw->sp_pwdp),spw->sp_pwdp) == 0)
                        {
                            pwd_good=1;
                        }
                        else
                        {
                            sprintf(m_menu.input.text,"密码错误！！！");
                            creat_button(disp,m_menu);
                        }
                    }
                    else
                    {
                        printf("%s\n",pw->pw_passwd);
                        printf("%s\n",crypt(m_menu.input.tag_text, pw->pw_passwd));
                        if (strcmp(crypt(m_menu.input.tag_text, pw->pw_passwd),pw->pw_passwd) == 0)
                        {
                            pwd_good=1;
                        }
                        else
                        {
                            sprintf(m_menu.input.text,"密码错误！！！");
                            creat_button(disp,m_menu);
                        }
                    }
                    if ((pwd_good==1) and (error==0))
                    {
                        char ch2[1024];
                        system("cp /etc/icon-de/icon-dm.rc /etc/icon-de/icon-dm.rc~");
                        if (access("/etc/icon-de/icon-dm.rc~",0)!=-1)
                        {
                            FILE * freeNum2;
                            freeNum=fopen("/etc/icon-de/icon-dm.rc~","r");
                            if ( freeNum==NULL)
                            { 
                                printf("error on open /etc/icon-de/icon-dm.rc\n"); 
                            }
                            else
                            {
                                freeNum2=fopen("/etc/icon-de/icon-dm.rc","w");
                                fgets(buffer_rc,1024,freeNum);
                                fputs(buffer_rc,freeNum2);
                                fgets(buffer_rc,1024,freeNum);
                                fputs(buffer_rc,freeNum2);
                                fgets(buffer_rc,1024,freeNum);
                                fputs(buffer_rc,freeNum2);
                                fgets(buffer_rc,1024,freeNum);
                                fputs(buffer_rc,freeNum2);
                                fgets(buffer_rc,1024,freeNum);
                                fputs(buffer_rc,freeNum2);

                                for (n=0;n<100;n++)
                                {
                                    if (m_menu.menu3[n].selected==1)
                                    {
                                        sprintf(ch2,"default_wm:%s\n",m_menu.menu3[n].text);
                                        fputs(ch2,freeNum2);
                                        break;
                                    }
                                }
                                for (n=100;n<500;n++)
                                {
                                    if (m_menu.menu3[n].selected==1)
                                    {
                                        sprintf(ch2,"default_usr:%s\n",m_menu.menu3[n].text);
                                        fputs(ch2,freeNum2);
                                        break;
                                    }
                                }
                                fclose(freeNum2);
                            }
                            fclose(freeNum);
                        }
                        r_flash=1;
                    }
                }
                else
                    input_text(disp,ev,m_menu);
                break;
            case LeaveNotify:        
                break;
            case ButtonPress:
                i=click_button(disp,ev.xmotion.window,m_menu);
                if (i==0)
                {
                    i=5;
//                    char passwd[1024];
                    for (n=100;n<500;n++)
                    {
                        if (m_menu.menu3[n].selected==1)
                            break;
                    }
//                    char *name, pwname[9];
                    int shadow = 0;
                    int error=0;
                    int pwd_good=0;
                    struct spwd *spw;
                    pw = getpwuid(m_menu.menu3[n].tag_int);
                    if (pw == NULL) {
                        error=1;
                    }
                    printf("Password:%s\n",pw->pw_passwd);
                    if ((pw->pw_passwd[0] == 'x') or (pw->pw_passwd[1] == 'x') )
                    {
                        shadow = 1;
                        spw=getspnam(pw->pw_name);
                        if (spw == NULL)
                            error=1;
                        else
                            printf("Password:%s\n",spw->sp_pwdp);
                    } 
                    if (shadow == 1)
                    {
                        printf("spw:%s|||\n",spw->sp_pwdp);
                        printf("spw:%s|||\n",crypt(m_menu.input.tag_text, spw->sp_pwdp));
                        if (strcmp(crypt(m_menu.input.tag_text, spw->sp_pwdp),spw->sp_pwdp) == 0)
                        {
                            pwd_good=1;
                        }
                        else
                        {
                            sprintf(m_menu.input.text,"密码错误！！！");
                            creat_button(disp,m_menu);
                        }
                    }
                    else
                    {
                        printf("%s\n",pw->pw_passwd);
                        printf("%s\n",crypt(m_menu.input.tag_text, pw->pw_passwd));
                        if (strcmp(crypt(m_menu.input.tag_text, pw->pw_passwd),pw->pw_passwd) == 0)
                        {
                            pwd_good=1;
                        }
                        else
                        {
                            sprintf(m_menu.input.text,"密码错误！！！");
                            creat_button(disp,m_menu);
                        }
                    }
                    if ((pwd_good==1) and (error==0))
                    {
                        char ch2[1024];
                        system("cp /etc/icon-de/icon-dm.rc /etc/icon-de/icon-dm.rc~");
                        if (access("/etc/icon-de/icon-dm.rc~",0)!=-1)
                        {
                            FILE * freeNum2;
                            freeNum=fopen("/etc/icon-de/icon-dm.rc~","r");
                            if ( freeNum==NULL)
                            { 
                                printf("error on open /etc/icon-de/icon-dm.rc\n"); 
                            }
                            else
                            {
                                freeNum2=fopen("/etc/icon-de/icon-dm.rc","w");
                                fgets(buffer_rc,1024,freeNum);
                                fputs(buffer_rc,freeNum2);
                                fgets(buffer_rc,1024,freeNum);
                                fputs(buffer_rc,freeNum2);
                                fgets(buffer_rc,1024,freeNum);
                                fputs(buffer_rc,freeNum2);
                                fgets(buffer_rc,1024,freeNum);
                                fputs(buffer_rc,freeNum2);
                                fgets(buffer_rc,1024,freeNum);
                                fputs(buffer_rc,freeNum2);

                                for (n=0;n<100;n++)
                                {
                                    if (m_menu.menu3[n].selected==1)
                                    {
                                        sprintf(ch2,"default_wm:%s\n",m_menu.menu3[n].text);
                                        fputs(ch2,freeNum2);
                                        break;
                                    }
                                }
                                for (n=100;n<500;n++)
                                {
                                    if (m_menu.menu3[n].selected==1)
                                    {
                                        sprintf(ch2,"default_usr:%s\n",m_menu.menu3[n].text);
                                        fputs(ch2,freeNum2);
                                        break;
                                    }
                                }
                                fclose(freeNum2);
                            }
                            fclose(freeNum);
                        }
                        r_flash=1;
                    }
                }
                else if (i==1)
                    system("halt &");
                else if (i==2)
                    system("reboot &");
                break;
            case DestroyNotify:
                break;
            }
        } 
        else
        {
            FD_ZERO(&rfds);
            FD_SET(xfd, &rfds);
            if (r_flash==1)
            {
                r_flash=0;
                for (n=0;n<100;n++)
                {
                    if (m_menu.menu3[n].selected==1)
                        break;
                }
                setenv("USER",pw->pw_name,1);
                setenv("LOGNAME",pw->pw_name,1);
                setenv("SHELL",pw->pw_shell,1);
                setenv("HOME",pw->pw_dir,1);

                if( !pw || initgroups(pw->pw_name, pw->pw_gid) ||
                    setgid(pw->pw_gid) || setuid(pw->pw_uid) || setsid() == -1 )
                    exit(EXIT_FAILURE);
                chdir(pw->pw_dir);
                char cmd[256];
                printf("icon-dm.sh %s\n",m_menu.menu3[n].text);
                sprintf(cmd,"icon-dm.sh %s",m_menu.menu3[n].text);
                setenv("SIG","BEGIN",1);
                pthread_t t1;
                int temp;
                if((temp = pthread_create(&t1,NULL,exe_cmd,(void *)cmd))!=0)
                    setenv("SIG","END",1);
            }
            if (strcmp(getenv("SIG"),"END")==0)
            {
                shutdown();
                sleep(3);
                initserver(argc,argv);
                waitforserver();
                creat_button(disp,m_menu);
            } 
            tm.tv_sec=0l;
            tm.tv_usec=400000l;
            select(xfd + 1, &rfds, 0, 0, &tm);
        } /* End else*/
    }
}


/*
 *    waitforserver - wait for X server to start up
 */
static Bool
waitforserver(void)
{
    int    ncycles     = 120;        /* # of cycles to wait */
    int    cycles;            /* Wait cycle count */

#ifdef __APPLE__
    /* For Apple, we don't get signaled by the server when it's ready, so we just
     * want to sleep now since we're going to sleep later anyways and this allows us
     * to avoid the awkard, "why is there an error message in the log" questions
     * from users.
     */

    sleep(2);
#endif

    for (cycles = 0; cycles < ncycles; cycles++) {
        if ((xd = XOpenDisplay(displayNum))) {
            return(TRUE);
        }
        else {
            if (!processTimeout(1, (char *)"X server to begin accepting connections"))
              break;
        }
    }

    Errorx("giving up");

    return(FALSE);
}

/*
 * return TRUE if we timeout waiting for pid to exit, FALSE otherwise.
 */
static Bool
processTimeout(int timeout, char *string)
{
    int    i = 0, pidfound = -1;
    static char    *laststring;

    for (;;) {
        if ((pidfound = waitpid(serverpid, &status, WNOHANG)) == serverpid)
            break;
        if (timeout) {
            if (i == 0 && string != laststring)
                fprintf(stderr, "\r\nwaiting for %s ", string);
            else
                fprintf(stderr, ".");
            fflush(stderr);
            sleep(1);
        }
        if (++i > timeout)
            break;
    }
    if (i > 0) fputc('\n', stderr);     /* tidy up after message */
    laststring = string;
    return (serverpid != pidfound);
}

static int
startServer(char *server[])
{
    sigset_t mask, old;
    const char * const *cpp;

    sigemptyset(&mask);
    sigaddset(&mask, SIGUSR1);
    sigprocmask(SIG_BLOCK, &mask, &old);

    serverpid = fork();

    switch(serverpid) {
    case 0:
        /* Unblock */
        sigprocmask(SIG_SETMASK, &old, NULL);

        /*
         * don't hang on read/write to control tty
         */
        signal(SIGTTIN, SIG_IGN);
        signal(SIGTTOU, SIG_IGN);
        /*
         * ignore SIGUSR1 in child.  The server
         * will notice this and send SIGUSR1 back
         * at xinit when ready to accept connections
         */
        signal(SIGUSR1, SIG_IGN);
        /*
         * prevent server from getting sighup from vhangup()
         * if client is xterm -L
         */
        setpgid(0,getpid());
        Execute(server);

        Error("unable to run server \"%s\"", server[0]);

        fprintf(stderr, "Use the -- option, or make sure that %s is in your path and\n", bindir);
        fprintf(stderr, "that \"%s\" is a program or a link to the right type of server\n", server[0]);
        fprintf(stderr, "for your display.  Possible server names include:\n\n");
        for (cpp = server_names; *cpp; cpp++)
            fprintf(stderr, "    %s\n", *cpp);
        fprintf(stderr, "\n");

        exit(EXIT_FAILURE);

        break;
    case -1:
        break;
    default:
        /*
         * don't nice server
         */
        setpriority(PRIO_PROCESS, serverpid, -1);

        errno = 0;
        if(! processTimeout(0, (char *)"")) {
            serverpid = -1;
            break;
        }
        /*
         * kludge to avoid race with TCP, giving server time to
         * set his socket options before we try to open it,
         * either use the 15 second timeout, or await SIGUSR1.
         *
         * If your machine is substantially slower than 15 seconds,
         * you can easily adjust this value.
         */
        alarm(15);

        sigsuspend(&old);
        alarm(0);
        sigprocmask(SIG_SETMASK, &old, NULL);

        if (waitforserver() == 0) {
            Error("unable to connect to X server");
            shutdown();
            serverpid = -1;
        }
        break;
    }

    return(serverpid);
}


static jmp_buf close_env;

static int
ignorexio(Display *dpy)
{
    Errorx("connection to X server lost");
    longjmp(close_env, 1);
    /*NOTREACHED*/
    return 0;
}

static void
shutdown(void)
{
    /* have kept display opened, so close it now */
    signal(SIGQUIT, SIG_IGN);
    signal(SIGINT, SIG_IGN);
    signal(SIGHUP, SIG_IGN);
    signal(SIGPIPE, SIG_IGN);
    signal(SIGTERM, SIG_DFL);
    signal(SIGKILL, SIG_DFL);
    signal(SIGALRM, SIG_DFL);
    if (clientpid > 0) {
        XSetIOErrorHandler(ignorexio);
        if (! setjmp(close_env)) {
            XCloseDisplay(xd);
        }

        /* HUP all local clients to allow them to clean up */
        if (killpg(clientpid, SIGHUP) < 0 && errno != ESRCH)
            Error("can't send HUP to process group %d", clientpid);
    }

    if (serverpid < 0)
        return;

    if (killpg(serverpid, SIGTERM) < 0) {
        if (errno == ESRCH)
            return;
        Fatal("can't kill X server");
    }

    if (!processTimeout(10, (char *)"X server to shut down"))
        return;

    Errorx("X server slow to shut down, sending KILL signal");

    if (killpg(serverpid, SIGKILL) < 0) {
        if (errno == ESRCH)
            return;
        Error("can't SIGKILL X server");
    }

    if (processTimeout(3, (char *)"server to die"))
        Fatalx("X server refuses to die");
}

static void
verror(const char *fmt, va_list ap)
{
    fprintf(stderr, "%s: ", program);
    vfprintf(stderr, fmt, ap);
    fprintf(stderr, ": %s\n", strerror(errno));
}

static void
verrorx(const char *fmt, va_list ap)
{
    fprintf(stderr, "%s: ", program);
    vfprintf(stderr, fmt, ap);
    fprintf(stderr, "\n");
}

static void
Fatal(const char *fmt, ...)
{
    va_list ap;
    va_start(ap, fmt);
    verror(fmt, ap);
    va_end(ap);
//    exit(EXIT_FAILURE);
}

static void
Fatalx(const char *fmt, ...)
{
    va_list ap;
    va_start(ap, fmt);
    verrorx(fmt, ap);
    va_end(ap);
//    exit(EXIT_FAILURE);
}

static void
Error(const char *fmt, ...)
{
    va_list ap;
    va_start(ap, fmt);
    verror(fmt, ap);
    va_end(ap);
}

static void
Errorx(const char *fmt, ...)
{
    va_list ap;
    va_start(ap, fmt);
    verrorx(fmt, ap);
    va_end(ap);
}
